@@ -8,7 +8,6 @@
 #include <ctype.h>
 #include <string.h>
 #include <errno.h>
-#include <getopt.h>
 #include <sys/ipc.h>
 #include <sys/shm.h>
 #include <sys/types.h>
@@ -21,13 +20,16 @@
 #include "verify.h"
 #include "profile.h"
 
-static char fio_version_string[] = "fio 1.38";
+#include "lib/getopt.h"
+
+static char fio_version_string[] = "fio 1.59";
 
 #define FIO_RANDSEED		(0xb1899bedUL)
 
 static char **ini_file;
-static int max_jobs = MAX_JOBS;
+static int max_jobs = FIO_MAX_JOBS;
 static int dump_cmdline;
+static int def_timeout;
 
 static struct thread_data def_thread;
 struct thread_data *threads = NULL;
@@ -38,13 +40,15 @@ int eta_print;
 unsigned long long mlock_size = 0;
 FILE *f_out = NULL;
 FILE *f_err = NULL;
-char *job_section = NULL;
+char **job_sections = NULL;
+int nr_job_sections = 0;
 char *exec_profile = NULL;
+int warnings_fatal = 0;
+int terse_version = 2;
 
 int write_bw_log = 0;
 int read_only = 0;
 
-static int def_timeout;
 static int write_lat_log;
 
 static int prev_group_jobs;
@@ -53,86 +57,103 @@ unsigned long fio_debug = 0;
 unsigned int fio_debug_jobno = -1;
 unsigned int *fio_debug_jobp = NULL;
 
+static char cmd_optstr[256];
+
 /*
  * Command line options. These will contain the above, plus a few
  * extra that only pertain to fio itself and not jobs.
  */
 static struct option l_opts[FIO_NR_OPTIONS] = {
 	{
-		.name		= "output",
+		.name		= (char *) "output",
 		.has_arg	= required_argument,
 		.val		= 'o',
 	},
 	{
-		.name		= "timeout",
+		.name		= (char *) "timeout",
 		.has_arg	= required_argument,
 		.val		= 't',
 	},
 	{
-		.name		= "latency-log",
+		.name		= (char *) "latency-log",
 		.has_arg	= required_argument,
 		.val		= 'l',
 	},
 	{
-		.name		= "bandwidth-log",
+		.name		= (char *) "bandwidth-log",
 		.has_arg	= required_argument,
 		.val		= 'b',
 	},
 	{
-		.name		= "minimal",
+		.name		= (char *) "minimal",
 		.has_arg	= optional_argument,
 		.val		= 'm',
 	},
 	{
-		.name		= "version",
+		.name		= (char *) "version",
 		.has_arg	= no_argument,
 		.val		= 'v',
 	},
 	{
-		.name		= "help",
+		.name		= (char *) "help",
 		.has_arg	= no_argument,
 		.val		= 'h',
 	},
 	{
-		.name		= "cmdhelp",
+		.name		= (char *) "cmdhelp",
 		.has_arg	= optional_argument,
 		.val		= 'c',
 	},
 	{
-		.name		= "showcmd",
+		.name		= (char *) "showcmd",
 		.has_arg	= no_argument,
 		.val		= 's',
 	},
 	{
-		.name		= "readonly",
+		.name		= (char *) "readonly",
 		.has_arg	= no_argument,
 		.val		= 'r',
 	},
 	{
-		.name		= "eta",
+		.name		= (char *) "eta",
 		.has_arg	= required_argument,
 		.val		= 'e',
 	},
 	{
-		.name		= "debug",
+		.name		= (char *) "debug",
 		.has_arg	= required_argument,
 		.val		= 'd',
 	},
 	{
-		.name		= "section",
+		.name		= (char *) "section",
 		.has_arg	= required_argument,
 		.val		= 'x',
 	},
 	{
-		.name		= "alloc-size",
+		.name		= (char *) "alloc-size",
 		.has_arg	= required_argument,
 		.val		= 'a',
 	},
 	{
-		.name		= "profile",
+		.name		= (char *) "profile",
 		.has_arg	= required_argument,
 		.val		= 'p',
 	},
+	{
+		.name		= (char *) "warnings-fatal",
+		.has_arg	= no_argument,
+		.val		= 'w',
+	},
+	{
+		.name		= (char *) "max-jobs",
+		.has_arg	= required_argument,
+		.val		= 'j',
+	},
+	{
+		.name		= (char *) "terse-version",
+		.has_arg	= required_argument,
+		.val		= 'V',
+	},
 	{
 		.name		= NULL,
 	},
@@ -157,8 +178,11 @@ static struct thread_data *get_new_job(int global, struct thread_data *parent)
 
 	if (global)
 		return &def_thread;
-	if (thread_number >= max_jobs)
+	if (thread_number >= max_jobs) {
+		log_err("error: maximum number of jobs (%d) reached.\n",
+				max_jobs);
 		return NULL;
+	}
 
 	td = &threads[thread_number++];
 	*td = *parent;
@@ -178,7 +202,7 @@ static void put_job(struct thread_data *td)
 {
 	if (td == &def_thread)
 		return;
-	
+
 	profile_td_exit(td);
 
 	if (td->error)
@@ -193,6 +217,8 @@ static int __setup_rate(struct thread_data *td, enum fio_ddir ddir)
 	unsigned int bs = td->o.min_bs[ddir];
 	unsigned long long bytes_per_sec;
 
+	assert(ddir_rw(ddir));
+
 	if (td->o.rate[ddir])
 		bytes_per_sec = td->o.rate[ddir];
 	else
@@ -234,6 +260,7 @@ static int fixed_block_size(struct thread_options *o)
 static int fixup_options(struct thread_data *td)
 {
 	struct thread_options *o = &td->o;
+	int ret = 0;
 
 #ifndef FIO_HAVE_PSHARED_MUTEX
 	if (!o->use_thread) {
@@ -241,6 +268,7 @@ static int fixup_options(struct thread_data *td)
 			 " mutexes, forcing use of threads. Use the 'thread'"
 			 " option to get rid of this warning.\n");
 		o->use_thread = 1;
+		ret = warnings_fatal;
 	}
 #endif
 
@@ -248,6 +276,7 @@ static int fixup_options(struct thread_data *td)
 		log_err("fio: read iolog overrides write_iolog\n");
 		free(o->write_iolog_file);
 		o->write_iolog_file = NULL;
+		ret = warnings_fatal;
 	}
 
 	/*
@@ -286,6 +315,7 @@ static int fixup_options(struct thread_data *td)
 	    !o->norandommap) {
 		log_err("fio: Any use of blockalign= turns off randommap\n");
 		o->norandommap = 1;
+		ret = warnings_fatal;
 	}
 
 	if (!o->file_size_high)
@@ -296,6 +326,7 @@ static int fixup_options(struct thread_data *td)
 		log_err("fio: norandommap given for variable block sizes, "
 			"verify disabled\n");
 		o->verify = VERIFY_NONE;
+		ret = warnings_fatal;
 	}
 	if (o->bs_unaligned && (o->odirect || td->io_ops->flags & FIO_RAWIO))
 		log_err("fio: bs_unaligned may not work with raw io\n");
@@ -337,28 +368,32 @@ static int fixup_options(struct thread_data *td)
 	    ((o->ratemin[0] + o->ratemin[1]) && (o->rate_iops_min[0] +
 		o->rate_iops_min[1]))) {
 		log_err("fio: rate and rate_iops are mutually exclusive\n");
-		return 1;
+		ret = 1;
 	}
 	if ((o->rate[0] < o->ratemin[0]) || (o->rate[1] < o->ratemin[1]) ||
 	    (o->rate_iops[0] < o->rate_iops_min[0]) ||
 	    (o->rate_iops[1] < o->rate_iops_min[1])) {
 		log_err("fio: minimum rate exceeds rate\n");
-		return 1;
+		ret = 1;
 	}
 
 	if (!o->timeout && o->time_based) {
 		log_err("fio: time_based requires a runtime/timeout setting\n");
 		o->time_based = 0;
+		ret = warnings_fatal;
 	}
 
 	if (o->fill_device && !o->size)
 		o->size = -1ULL;
 
-	if (td_rw(td) && o->verify != VERIFY_NONE)
-		log_info("fio: mixed read/write workload with verify. May not "
-		 "work as expected, unless you pre-populated the file\n");
-
 	if (o->verify != VERIFY_NONE) {
+		if (td_write(td) && o->do_verify && o->numjobs > 1) {
+			log_info("Multiple writers may overwrite blocks that "
+				"belong to other jobs. This can cause "
+				"verification failures.\n");
+			ret = warnings_fatal;
+		}
+
 		o->refill_buffers = 1;
 		if (o->max_bs[DDIR_WRITE] != o->min_bs[DDIR_WRITE] &&
 		    !o->verify_interval)
@@ -367,9 +402,11 @@ static int fixup_options(struct thread_data *td)
 
 	if (o->pre_read) {
 		o->invalidate_cache = 0;
-		if (td->io_ops->flags & FIO_PIPEIO)
+		if (td->io_ops->flags & FIO_PIPEIO) {
 			log_info("fio: cannot pre-read files with an IO engine"
 				 " that isn't seekable. Pre-read disabled.\n");
+			ret = warnings_fatal;
+		}
 	}
 
 #ifndef FIO_HAVE_FDATASYNC
@@ -380,10 +417,11 @@ static int fixup_options(struct thread_data *td)
 			 " this warning\n");
 		o->fsync_blocks = o->fdatasync_blocks;
 		o->fdatasync_blocks = 0;
+		ret = warnings_fatal;
 	}
 #endif
 
-	return 0;
+	return ret;
 }
 
 /*
@@ -428,13 +466,15 @@ static int exists_and_not_file(const char *filename)
 	if (lstat(filename, &sb) == -1)
 		return 0;
 
-	if (S_ISREG(sb.st_mode))
+	/* \\.\ is the device namespace in Windows, where every file
+	 * is a device node */
+	if (S_ISREG(sb.st_mode) && strncmp(filename, "\\\\.\\", 4) != 0)
 		return 0;
 
 	return 1;
 }
 
-void td_fill_rand_seeds(struct thread_data *td)
+static void td_fill_rand_seeds_os(struct thread_data *td)
 {
 	os_random_seed(td->rand_seeds[0], &td->bsrange_state);
 	os_random_seed(td->rand_seeds[1], &td->verify_state);
@@ -444,6 +484,7 @@ void td_fill_rand_seeds(struct thread_data *td)
 		os_random_seed(td->rand_seeds[3], &td->next_file_state);
 
 	os_random_seed(td->rand_seeds[5], &td->file_size_state);
+	os_random_seed(td->rand_seeds[6], &td->trim_state);
 
 	if (!td_random(td))
 		return;
@@ -454,6 +495,37 @@ void td_fill_rand_seeds(struct thread_data *td)
 	os_random_seed(td->rand_seeds[4], &td->random_state);
 }
 
+static void td_fill_rand_seeds_internal(struct thread_data *td)
+{
+	init_rand_seed(&td->__bsrange_state, td->rand_seeds[0]);
+	init_rand_seed(&td->__verify_state, td->rand_seeds[1]);
+	init_rand_seed(&td->__rwmix_state, td->rand_seeds[2]);
+
+	if (td->o.file_service_type == FIO_FSERVICE_RANDOM)
+		init_rand_seed(&td->__next_file_state, td->rand_seeds[3]);
+
+	init_rand_seed(&td->__file_size_state, td->rand_seeds[5]);
+	init_rand_seed(&td->__trim_state, td->rand_seeds[6]);
+
+	if (!td_random(td))
+		return;
+
+	if (td->o.rand_repeatable)
+		td->rand_seeds[4] = FIO_RANDSEED * td->thread_number;
+
+	init_rand_seed(&td->__random_state, td->rand_seeds[4]);
+}
+
+void td_fill_rand_seeds(struct thread_data *td)
+{
+	if (td->o.use_os_rand)
+		td_fill_rand_seeds_os(td);
+	else
+		td_fill_rand_seeds_internal(td);
+
+	init_rand_seed(&td->buf_state, td->rand_seeds[7]);
+}
+
 /*
  * Initialize the various random states we need (random io, block size ranges,
  * read/write mix, etc).
@@ -509,7 +581,7 @@ static int add_job(struct thread_data *td, const char *jobname, int job_add_num)
 	}
 
 	if (profile_td_init(td))
-		return 1;
+		goto err;
 
 	engine = get_engine_name(td->o.ioengine);
 	td->io_ops = load_ioengine(td, engine);
@@ -553,10 +625,17 @@ static int add_job(struct thread_data *td, const char *jobname, int job_add_num)
 
 	td->mutex = fio_mutex_init(0);
 
+	td->ts.clat_percentiles = td->o.clat_percentiles;
+	if (td->o.overwrite_plist)
+		td->ts.percentile_list = td->o.percentile_list;
+	else
+		td->ts.percentile_list = NULL;
+
 	td->ts.clat_stat[0].min_val = td->ts.clat_stat[1].min_val = ULONG_MAX;
 	td->ts.slat_stat[0].min_val = td->ts.slat_stat[1].min_val = ULONG_MAX;
+	td->ts.lat_stat[0].min_val = td->ts.lat_stat[1].min_val = ULONG_MAX;
 	td->ts.bw_stat[0].min_val = td->ts.bw_stat[1].min_val = ULONG_MAX;
-	td->ddir_nr = td->o.ddir_nr;
+	td->ddir_seq_nr = td->o.ddir_seq_nr;
 
 	if ((td->o.stonewall || td->o.new_group) && prev_group_jobs) {
 		prev_group_jobs = 0;
@@ -573,6 +652,7 @@ static int add_job(struct thread_data *td, const char *jobname, int job_add_num)
 		goto err;
 
 	if (td->o.write_lat_log) {
+		setup_log(&td->ts.lat_log);
 		setup_log(&td->ts.slat_log);
 		setup_log(&td->ts.clat_log);
 	}
@@ -687,12 +767,18 @@ void add_job_opts(const char **o)
 
 static int skip_this_section(const char *name)
 {
-	if (!job_section)
+	int i;
+
+	if (!nr_job_sections)
 		return 0;
 	if (!strncmp(name, "global", 6))
 		return 0;
 
-	return strcmp(job_section, name);
+	for (i = 0; i < nr_job_sections; i++)
+		if (!strcmp(job_sections[i], name))
+			return 0;
+
+	return 1;
 }
 
 static int is_empty_or_comment(char *line)
@@ -704,7 +790,7 @@ static int is_empty_or_comment(char *line)
 			return 1;
 		if (line[i] == '#')
 			return 1;
-		if (!isspace(line[i]) && !iscntrl(line[i]))
+		if (!isspace((int) line[i]) && !iscntrl((int) line[i]))
 			return 0;
 	}
 
@@ -768,7 +854,7 @@ static int parse_jobs_ini(char *file, int stonewall_flag)
 
 		if (is_empty_or_comment(p))
 			continue;
-		if (sscanf(p, "[%255s]", name) != 1) {
+		if (sscanf(p, "[%255[^\n]]", name) != 1) {
 			if (inside_skip)
 				continue;
 			log_err("fio: option <%s> outside of [] job section\n",
@@ -776,6 +862,8 @@ static int parse_jobs_ini(char *file, int stonewall_flag)
 			break;
 		}
 
+		name[strlen(name) - 1] = '\0';
+
 		if (skip_this_section(name)) {
 			inside_skip = 1;
 			continue;
@@ -784,8 +872,6 @@ static int parse_jobs_ini(char *file, int stonewall_flag)
 
 		global = !strncmp(name, "global", 6);
 
-		name[strlen(name) - 1] = '\0';
-
 		if (dump_cmdline) {
 			if (first_sect)
 				log_info("fio ");
@@ -874,16 +960,12 @@ static int fill_def_thread(void)
 	memset(&def_thread, 0, sizeof(def_thread));
 
 	fio_getaffinity(getpid(), &def_thread.o.cpumask);
+	def_thread.o.timeout = def_timeout;
 
 	/*
 	 * fill default options
 	 */
 	fio_fill_default_options(&def_thread);
-
-	def_thread.o.timeout = def_timeout;
-	def_thread.o.write_bw_log = write_bw_log;
-	def_thread.o.write_lat_log = write_lat_log;
-
 	return 0;
 }
 
@@ -964,6 +1046,7 @@ static void usage(const char *name)
 	printf("\t--bandwidth-log\tGenerate per-job bandwidth logs\n");
 	printf("\t--minimal\tMinimal (terse) output\n");
 	printf("\t--version\tPrint version info and exit\n");
+	printf("\t--terse-version=x Terse version output format\n");
 	printf("\t--help\t\tPrint this page\n");
 	printf("\t--cmdhelp=cmd\tPrint command help, \"all\" for all of"
 		" them\n");
@@ -975,7 +1058,10 @@ static void usage(const char *name)
 	printf("\t--section=name\tOnly run specified section in job file\n");
 	printf("\t--alloc-size=kb\tSet smalloc pool to this size in kb"
 		" (def 1024)\n");
-	printf("\nFio was written by Jens Axboe <jens.axboe@oracle.com>\n");
+	printf("\t--warnings-fatal Fio parser warnings are fatal\n");
+	printf("\t--max-jobs\tMaximum number of threads/processes to support\n");
+	printf("\nFio was written by Jens Axboe <jens.axboe@oracle.com>");
+	printf("\n                   Jens Axboe <jaxboe@fusionio.com>\n");
 }
 
 #ifdef FIO_INC_DEBUG
@@ -992,6 +1078,7 @@ struct debug_level debug_levels[] = {
 	{ .name = "job",	.shift = FD_JOB },
 	{ .name = "mutex",	.shift = FD_MUTEX },
 	{ .name	= "profile",	.shift = FD_PROFILE },
+	{ .name = "time",	.shift = FD_TIME },
 	{ .name = NULL, },
 };
 
@@ -1003,8 +1090,6 @@ static int set_debug(const char *string)
 	int i;
 
 	if (!strcmp(string, "?") || !strcmp(string, "help")) {
-		int i;
-
 		log_info("fio: dumping debug options:");
 		for (i = 0; debug_levels[i].name; i++) {
 			dl = &debug_levels[i];
@@ -1059,12 +1144,32 @@ static int set_debug(const char *string)
 }
 #endif
 
+static void fio_options_fill_optstring(void)
+{
+	char *ostr = cmd_optstr;
+	int i, c;
+
+	c = i = 0;
+	while (l_opts[i].name) {
+		ostr[c++] = l_opts[i].val;
+		if (l_opts[i].has_arg == required_argument)
+			ostr[c++] = ':';
+		else if (l_opts[i].has_arg == optional_argument) {
+			ostr[c++] = ':';
+			ostr[c++] = ':';
+		}
+		i++;
+	}
+	ostr[c] = '\0';
+}
+
 static int parse_cmd_line(int argc, char *argv[])
 {
 	struct thread_data *td = NULL;
 	int c, ini_idx = 0, lidx, ret = 0, do_exit = 0, exit_val = 0;
+	char *ostr = cmd_optstr;
 
-	while ((c = getopt_long_only(argc, argv, "", l_opts, &lidx)) != -1) {
+	while ((c = getopt_long_only(argc, argv, ostr, l_opts, &lidx)) != -1) {
 		switch (c) {
 		case 'a':
 			smalloc_pool_size = atoi(optarg);
@@ -1075,7 +1180,7 @@ static int parse_cmd_line(int argc, char *argv[])
 		case 'l':
 			write_lat_log = 1;
 			break;
-		case 'w':
+		case 'b':
 			write_bw_log = 1;
 			break;
 		case 'o':
@@ -1101,8 +1206,16 @@ static int parse_cmd_line(int argc, char *argv[])
 			read_only = 1;
 			break;
 		case 'v':
-			printf("%s\n", fio_version_string);
+			log_info("%s\n", fio_version_string);
 			exit(0);
+		case 'V':
+			terse_version = atoi(optarg);
+			if (terse_version != 2) {
+				log_err("fio: bad terse version format\n");
+				exit_val = 1;
+				do_exit++;
+			}
+			break;
 		case 'e':
 			if (!strcmp("always", optarg))
 				eta_print = FIO_ETA_ALWAYS;
@@ -1113,7 +1226,9 @@ static int parse_cmd_line(int argc, char *argv[])
 			if (set_debug(optarg))
 				do_exit++;
 			break;
-		case 'x':
+		case 'x': {
+			size_t new_size;
+
 			if (!strcmp(optarg, "global")) {
 				log_err("fio: can't use global as only "
 					"section\n");
@@ -1121,10 +1236,12 @@ static int parse_cmd_line(int argc, char *argv[])
 				exit_val = 1;
 				break;
 			}
-			if (job_section)
-				free(job_section);
-			job_section = strdup(optarg);
+			new_size = (nr_job_sections + 1) * sizeof(char *);
+			job_sections = realloc(job_sections, new_size);
+			job_sections[nr_job_sections] = strdup(optarg);
+			nr_job_sections++;
 			break;
+			}
 		case 'p':
 			exec_profile = strdup(optarg);
 			break;
@@ -1134,10 +1251,8 @@ static int parse_cmd_line(int argc, char *argv[])
 
 			if (!strncmp(opt, "name", 4) && td) {
 				ret = add_job(td, td->o.name ?: "fio", 0);
-				if (ret) {
-					put_job(td);
+				if (ret)
 					return 0;
-				}
 				td = NULL;
 			}
 			if (!td) {
@@ -1158,6 +1273,17 @@ static int parse_cmd_line(int argc, char *argv[])
 			ret = fio_cmd_option_parse(td, opt, val);
 			break;
 		}
+		case 'w':
+			warnings_fatal = 1;
+			break;
+		case 'j':
+			max_jobs = atoi(optarg);
+			if (!max_jobs || max_jobs > REAL_MAX_JOBS) {
+				log_err("fio: invalid max jobs: %d\n", max_jobs);
+				do_exit++;
+				exit_val = 1;
+			}
+			break;
 		default:
 			do_exit++;
 			exit_val = 1;
@@ -1171,8 +1297,6 @@ static int parse_cmd_line(int argc, char *argv[])
 	if (td) {
 		if (!ret)
 			ret = add_job(td, td->o.name ?: "fio", 0);
-		if (ret)
-			put_job(td);
 	}
 
 	while (optind < argc) {
@@ -1192,6 +1316,7 @@ int parse_options(int argc, char *argv[])
 	f_out = stdout;
 	f_err = stderr;
 
+	fio_options_fill_optstring();
 	fio_options_dup_and_init(l_opts);
 
 	if (setup_thread_area())
@@ -1218,7 +1343,7 @@ int parse_options(int argc, char *argv[])
 		if (exec_profile)
 			return 0;
 
-		log_err("No jobs defined(s)\n\n");
+		log_err("No jobs(s) defined\n\n");
 		usage(argv[0]);
 		return 1;
 	}
@@ -1229,5 +1354,6 @@ int parse_options(int argc, char *argv[])
 		fio_gtod_cpu = def_thread.o.gtod_cpu;
 	}
 
+	log_info("%s\n", fio_version_string);
 	return 0;
 }
