@@ -8,6 +8,7 @@ enum fio_ddir {
 	DDIR_DATASYNC,
 	DDIR_SYNC_FILE_RANGE,
 	DDIR_WAIT,
+	DDIR_TRIM,
 	DDIR_INVAL = -1,
 };
 
@@ -33,4 +34,9 @@ static inline int ddir_sync(enum fio_ddir ddir)
 	       ddir == DDIR_SYNC_FILE_RANGE;
 }
 
+static inline int ddir_rw(enum fio_ddir ddir)
+{
+	return ddir == DDIR_READ || ddir == DDIR_WRITE;
+}
+
 #endif
