@@ -14,6 +14,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <dlfcn.h>
+#include <fcntl.h>
 #include <assert.h>
 
 #include "fio.h"
@@ -207,6 +208,9 @@ int td_io_getevents(struct thread_data *td, unsigned int min, unsigned int max,
 out:
 	if (r >= 0)
 		io_u_mark_complete(td, r);
+	else
+		td_verror(td, r, "get_events");
+
 	dprint(FD_IO, "getevents: %d\n", r);
 	return r;
 }
@@ -238,7 +242,7 @@ int td_io_queue(struct thread_data *td, struct io_u *io_u)
 					sizeof(struct timeval));
 	}
 
-	if (!ddir_sync(io_u->ddir))
+	if (ddir_rw(io_u->ddir))
 		td->io_issues[io_u->ddir]++;
 
 	ret = td->io_ops->queue(td, io_u);
@@ -251,7 +255,7 @@ int td_io_queue(struct thread_data *td, struct io_u *io_u)
 	 * IO, then it's likely an alignment problem or because the host fs
 	 * does not support O_DIRECT
 	 */
-	if (io_u->error == EINVAL && td->io_issues[io_u->ddir] == 1 &&
+	if (io_u->error == EINVAL && td->io_issues[io_u->ddir & 1] == 1 &&
 	    td->o.odirect) {
 		log_info("fio: first direct IO errored. File system may not "
 			 "support direct IO, or iomem_align= is bad.\n");
@@ -263,14 +267,15 @@ int td_io_queue(struct thread_data *td, struct io_u *io_u)
 	}
 
 	if (ret == FIO_Q_COMPLETED) {
-		if (!ddir_sync(io_u->ddir)) {
+		if (ddir_rw(io_u->ddir)) {
 			io_u_mark_depth(td, 1);
 			td->ts.total_io_u[io_u->ddir]++;
-		}
+		} else if (io_u->ddir == DDIR_TRIM)
+			td->ts.total_io_u[2]++;
 	} else if (ret == FIO_Q_QUEUED) {
 		int r;
 
-		if (!ddir_sync(io_u->ddir)) {
+		if (ddir_rw(io_u->ddir)) {
 			td->io_u_queued++;
 			td->ts.total_io_u[io_u->ddir]++;
 		}
@@ -307,6 +312,8 @@ int td_io_init(struct thread_data *td)
 			log_err("fio: io engine init failed. Perhaps try"
 				" reducing io depth?\n");
 		}
+		if (!td->error)
+			td->error = ret;
 	}
 
 	return ret;
@@ -314,6 +321,8 @@ int td_io_init(struct thread_data *td)
 
 int td_io_commit(struct thread_data *td)
 {
+	int ret;
+
 	dprint(FD_IO, "calling ->commit(), depth %d\n", td->cur_depth);
 
 	if (!td->cur_depth || !td->io_u_queued)
@@ -322,8 +331,11 @@ int td_io_commit(struct thread_data *td)
 	io_u_mark_depth(td, td->io_u_queued);
 	td->io_u_queued = 0;
 
-	if (td->io_ops->commit)
-		return td->io_ops->commit(td);
+	if (td->io_ops->commit) {
+		ret = td->io_ops->commit(td);
+		if (ret)
+			td_verror(td, -ret, "io commit");
+	}
 
 	return 0;
 }
@@ -377,7 +389,7 @@ int td_io_open_file(struct thread_data *td, struct fio_file *f)
 		else
 			flags = POSIX_FADV_SEQUENTIAL;
 
-		if (fadvise(f->fd, f->file_offset, f->io_size, flags) < 0) {
+		if (posix_fadvise(f->fd, f->file_offset, f->io_size, flags) < 0) {
 			td_verror(td, errno, "fadvise");
 			goto err;
 		}
@@ -393,6 +405,7 @@ int td_io_open_file(struct thread_data *td, struct fio_file *f)
 
 		if (ret) {
 			td_verror(td, ret, "fio_set_odirect");
+			log_err("fio: the file system does not seem to support direct IO\n");
 			goto err;
 		}
 	}
@@ -431,3 +444,60 @@ int td_io_get_file_size(struct thread_data *td, struct fio_file *f)
 
 	return td->io_ops->get_file_size(td, f);
 }
+
+static int do_sync_file_range(struct thread_data *td, struct fio_file *f)
+{
+	off64_t offset, nbytes;
+
+	offset = f->first_write;
+	nbytes = f->last_write - f->first_write;
+
+	if (!nbytes)
+		return 0;
+
+	return sync_file_range(f->fd, offset, nbytes, td->o.sync_file_range);
+}
+
+int do_io_u_sync(struct thread_data *td, struct io_u *io_u)
+{
+	int ret;
+
+	if (io_u->ddir == DDIR_SYNC) {
+		ret = fsync(io_u->file->fd);
+	} else if (io_u->ddir == DDIR_DATASYNC) {
+#ifdef FIO_HAVE_FDATASYNC
+		ret = fdatasync(io_u->file->fd);
+#else
+		ret = io_u->xfer_buflen;
+		io_u->error = EINVAL;
+#endif
+	} else if (io_u->ddir == DDIR_SYNC_FILE_RANGE)
+		ret = do_sync_file_range(td, io_u->file);
+	else {
+		ret = io_u->xfer_buflen;
+		io_u->error = EINVAL;
+	}
+
+	if (ret < 0)
+		io_u->error = errno;
+
+	return ret;
+}
+
+int do_io_u_trim(struct thread_data *td, struct io_u *io_u)
+{
+#ifndef FIO_HAVE_TRIM
+	io_u->error = EINVAL;
+	return 0;
+#else
+	struct fio_file *f = io_u->file;
+	int ret;
+
+	ret = os_trim(f->fd, io_u->offset, io_u->xfer_buflen);
+	if (!ret)
+		return io_u->xfer_buflen;;
+
+	io_u->error = ret;
+	return 0;
+#endif
+}
