@@ -2,16 +2,26 @@
 #define FIO_OS_H
 
 #include <sys/types.h>
+#include <pthread.h>
 #include <unistd.h>
+#include <stdlib.h>
 
 #if defined(__linux__)
 #include "os-linux.h"
 #elif defined(__FreeBSD__)
 #include "os-freebsd.h"
+#elif defined(__NetBSD__)
+#include "os-netbsd.h"
 #elif defined(__sun__)
 #include "os-solaris.h"
 #elif defined(__APPLE__)
 #include "os-mac.h"
+#elif defined(_AIX)
+#include "os-aix.h"
+#elif defined(__hpux)
+#include "os-hpux.h"
+#elif defined(__CYGWIN__)
+#include "os-windows.h"
 #else
 #error "unsupported os"
 #endif
@@ -22,6 +32,9 @@
 
 #ifdef FIO_HAVE_POSIXAIO
 #include <aio.h>
+#ifndef FIO_OS_HAVE_AIOCB_TYPEDEF
+typedef struct aiocb os_aiocb_t;
+#endif
 #endif
 
 #ifdef FIO_HAVE_SGIO
@@ -33,8 +46,12 @@
 #include "../lib/strsep.h"
 #endif
 
+#ifdef MSG_DONTWAIT
+#define OS_MSG_DONTWAIT	MSG_DONTWAIT
+#endif
+
 #ifndef FIO_HAVE_FADVISE
-#define fadvise(fd, off, len, advice)	(0)
+#define posix_fadvise(fd, off, len, advice)	(0)
 
 #ifndef POSIX_FADV_DONTNEED
 #define POSIX_FADV_DONTNEED	(0)
@@ -48,6 +65,7 @@
 #define fio_getaffinity(pid, mask)	do { } while (0)
 #define fio_cpu_clear(mask, cpu)	do { } while (0)
 #define fio_cpuset_exit(mask)		(-1)
+typedef unsigned long os_cpu_mask_t;
 #endif
 
 #ifndef FIO_HAVE_IOPRIO
@@ -79,10 +97,24 @@
 #define OS_RAND_MAX			RAND_MAX
 #endif
 
+#ifdef FIO_HAVE_CLOCK_MONOTONIC
+#define FIO_TIMER_CLOCK CLOCK_MONOTONIC
+#else
+#define FIO_TIMER_CLOCK CLOCK_REALTIME
+#endif
+
 #ifndef FIO_HAVE_RAWBIND
 #define fio_lookup_raw(dev, majdev, mindev)	1
 #endif
 
+#ifndef FIO_PREFERRED_ENGINE
+#define FIO_PREFERRED_ENGINE	"sync"
+#endif
+
+#ifndef FIO_MAX_JOBS
+#define FIO_MAX_JOBS		2048
+#endif
+
 #ifndef FIO_HAVE_BLKTRACE
 static inline int is_blktrace(const char *fname)
 {
@@ -112,13 +144,13 @@ static inline int os_cache_line_size(void)
 }
 
 #ifdef FIO_USE_GENERIC_BDEV_SIZE
-static inline int blockdev_size(int fd, unsigned long long *bytes)
+static inline int blockdev_size(struct fio_file *f, unsigned long long *bytes)
 {
 	off_t end;
 
 	*bytes = 0;
 
-	end = lseek(fd, 0, SEEK_END);
+	end = lseek(f->fd, 0, SEEK_END);
 	if (end < 0)
 		return errno;
 
@@ -144,4 +176,25 @@ static inline long os_random_long(os_random_state_t *rs)
 }
 #endif
 
+#ifndef FIO_HAVE_FS_STAT
+static inline unsigned long long get_fs_size(const char *path)
+{
+	return 0;
+}
+#endif
+
+#ifndef FIO_HAVE_CPU_ONLINE_SYSCONF
+static inline unsigned int cpus_online(void)
+{
+	return sysconf(_SC_NPROCESSORS_ONLN);
+}
+#endif
+
+#ifndef FIO_HAVE_GETTID
+static inline int gettid(void)
+{
+	return getpid();
+}
+#endif
+
 #endif
